/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_COMMON_WL_BUFFER_H_
#define SCREEN_CONNECTOR_COMMON_WL_BUFFER_H_

#include <Ecore_Wl2.h>
#include <stdint.h>
#include <tbm_surface.h>
#include <tbm_surface_internal.h>
#include <tizen-remote-surface-client-protocol.h>
#include <unistd.h>
#include <wayland-extension/tizen-extension-client-protocol.h>

#include "screen_connector_common/export.h"
#include "screen_connector_common/handle.h"

namespace screen_connector {

class EXPORT WlBuffer : public Handle<struct wl_buffer*> {
 public:
  WlBuffer(struct wl_buffer* raw, struct tizen_remote_surface* trs)
    : Handle<struct wl_buffer*>(raw, true),
      trs_(trs),
      tbm_surface_info_(nullptr, std::free) { }

  virtual ~WlBuffer() {
    if (GetRaw() != nullptr) {
      tizen_remote_surface_release(trs_, GetRaw());
      tbm_surface_h tbm_surface = static_cast<tbm_surface_h>(
          wl_buffer_get_user_data(GetRaw()));

      if (tbm_surface_info_.get() != nullptr)
        tbm_surface_unmap(tbm_surface);

      tbm_surface_internal_unref(tbm_surface);
      wl_buffer_destroy(GetRaw());
    }
  }

  tbm_surface_info_s* GetTbmSurfaceInfo() {
    if (tbm_surface_info_ != nullptr)
      return tbm_surface_info_.get();

    tbm_surface_info_s* surface_info = static_cast<tbm_surface_info_s*>(
        calloc(1, sizeof(tbm_surface_info_s)));
    if (surface_info == nullptr)
      return nullptr;

    tbm_surface_h tbm_surface = static_cast<tbm_surface_h>(
        wl_buffer_get_user_data(GetRaw()));
    tbm_surface_map(tbm_surface, TBM_SURF_OPTION_READ, surface_info);
    tbm_surface_info_.reset(surface_info);

    return tbm_surface_info_.get();
  }

 private:
  struct tizen_remote_surface* trs_;
  std::unique_ptr<tbm_surface_info_s, decltype(std::free)*> tbm_surface_info_;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_COMMON_WL_BUFFER_H_
