/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_LAUNCHER_SERVICE_LAUNCHER_SERVICE_H_
#define SCREEN_CONNECTOR_LAUNCHER_SERVICE_LAUNCHER_SERVICE_H_

#include <memory>
#include <string>

#include "screen_connector_common/export.h"
#include "screen_connector_common/wl2_window.h"
#include "screen_connector_launcher_service/error.h"
#include "screen_connector_launcher_service/raw_image.h"

namespace screen_connector {

class EXPORT LauncherService {
 public:
  class EXPORT IEventListener {
   public:
    virtual void OnPrepareEvent(std::shared_ptr<RawImage> raw_image,
                                uint32_t serial) = 0;
    virtual void OnStopEvent(uint32_t serial) = 0;
    virtual void OnErrorEvent(Error error,
                              uint32_t serial) = 0;
    virtual void OnCleanUpEvent(uint32_t serial) = 0;
    virtual void OnResetEvent(uint32_t serial) = 0;
  };

  LauncherService(std::shared_ptr<Wl2Window> wl2_win,
                  IEventListener* listener);
  virtual ~LauncherService();

  int Launch(const std::string& app_id,
             const std::string& inst_id,
             int pid,
             uint32_t serial);
  int LaunchWithSharedWidget(const std::string& app_id,
                             const std::string& inst_id,
                             int pid,
                             uint32_t serial);
  int Launching(uint32_t serial);
  int LaunchCancel(uint32_t serial);
  int LaunchDone(uint32_t serial);

 private:
  class Impl;
  std::unique_ptr<Impl> impl_;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_LAUNCHER_SERVICE_LAUNCHER_SERVICE_H_
