/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_LAUNCHER_SERVICE_LAUNCHER_SERVICE_IMPLEMENTATION_H_
#define SCREEN_CONNECTOR_LAUNCHER_SERVICE_LAUNCHER_SERVICE_IMPLEMENTATION_H_

#include <stdint.h>

#include <memory>
#include <string>

#include "screen_connector_common/handle.h"
#include "screen_connector_common/tzsh.h"
#include "screen_connector_common/wl_buffer.h"
#include "screen_connector_common/wl_surface.h"
#include "screen_connector_launcher_service/launcher_service.h"
#include "screen_connector_launcher_service/tizen_remote_surface_internal.h"
#include "screen_connector_launcher_service/tizen_remote_surface_manager_internal.h"
#include "screen_connector_launcher_service/tzsh_launcher_service_internal.h"

namespace screen_connector {

class LauncherService::Impl : TizenRemoteSurface::IEventListener,
                              TzshLauncherService::IEventListener {
 public:
  // TizenRemoteSurface::IEventListener
  void OnRawBufferUpdated(std::shared_ptr<WlBuffer> wl_buffer,
                          uint32_t time,
                          uint32_t serial) override;
  void OnRawBufferMissing(uint32_t serial) override;
  void OnRawBufferChanged(uint32_t type,
                          std::shared_ptr<WlBuffer> wl_buffer,
                          int32_t fd,
                          uint32_t size,
                          uint32_t time,
                          Direction direction,
                          int32_t x,
                          int32_t y,
                          const std::string& shared_widget_info,
                          uint32_t serial) override;
  void OnInputFilterChanged(uint32_t event_filter,
                            uint32_t serial) override;

  // TzshLauncherService::IEventListener
  void OnPrepareEvent(uint32_t target_type,
                      uint32_t target_res_id,
                      const std::string& file_path,
                      const std::string& file_group,
                      Direction direction,
                      int32_t x,
                      int32_t y,
                      const std::string& shared_widget_info,
                      uint32_t serial) override;
  void OnStopEvent(uint32_t serial) override;
  void OnErrorEvent(Error error,
                    uint32_t serial) override;
  void OnCleanupEvent(uint32_t serial) override;

  virtual ~Impl() = default;

  int Launch(const std::string& app_id,
             const std::string& inst_id,
             int pid,
             uint32_t serial);
  int LaunchWithSharedWidget(const std::string& app_id,
                             const std::string& inst_id,
                             int pid,
                             uint32_t serial);
  int Launching(uint32_t serial);
  int LaunchCancel(uint32_t serial);
  int LaunchDone(uint32_t serial);

 private:
  friend class LauncherService;

  Impl(LauncherService* parent,
       std::shared_ptr<Wl2Window> wl2_win,
       LauncherService::IEventListener* listener);

 private:
  LauncherService* parent_;
  std::shared_ptr<Wl2Window> wl2_win_;
  LauncherService::IEventListener* listener_;
  std::unique_ptr<TizenRemoteSurfaceManager> trs_mgr_;
  std::unique_ptr<Tzsh> tzsh_;
  std::unique_ptr<TzshLauncherService> tls_;
  std::shared_ptr<WlSurface> wl_surface_;
  std::unique_ptr<TizenRemoteSurface> trs_;
  std::shared_ptr<RawImage> raw_image_;
  bool done_ = false;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_LAUNCHER_SERVICE_LAUNCHER_SERVICE_IMPLEMENTATION_H_
