/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_LAUNCHER_SERVICE_TZSH_LAUNCHER_SERVICE_INTERNAL_H_
#define SCREEN_CONNECTOR_LAUNCHER_SERVICE_TZSH_LAUNCHER_SERVICE_INTERNAL_H_

#include <stdint.h>
#include <tzsh_launcher_service.h>
#include <unistd.h>

#include <memory>
#include <string>

#include "screen_connector_common/handle.h"
#include "screen_connector_launcher_service/direction.h"
#include "screen_connector_launcher_service/error.h"

namespace screen_connector {

class TzshLauncherService : public Handle<tzsh_launcher_service_h> {
 public:
  class IEventListener {
   public:
    virtual void OnPrepareEvent(uint32_t target_type,
                                uint32_t target_res_id,
                                const std::string& file_path,
                                const std::string& file_group,
                                Direction direction,
                                int32_t x,
                                int32_t y,
                                const std::string& shared_widget_info,
                                uint32_t serial) = 0;
    virtual void OnStopEvent(uint32_t serial) = 0;
    virtual void OnErrorEvent(Error error,
                              uint32_t serial) = 0;
    virtual void OnCleanupEvent(uint32_t serial) = 0;
  };

  explicit TzshLauncherService(tzsh_launcher_service_h raw);
  virtual ~TzshLauncherService();

  tzsh_launcher_service_h GetHandle() const;
  void SetListener(IEventListener* listener);

  int Launch(const std::string& app_id,
             const std::string& inst_id,
             int pid,
             uint32_t serial);
  int LaunchWithSharedWidget(const std::string& app_id,
                             const std::string& inst_id,
                             int pid,
                             uint32_t serial);
  int Launching(uint32_t serial);
  int LaunchCancel(uint32_t serial);
  int LaunchDone(uint32_t serial);

 private:
  static void PrepareCB(void* data,
                        tzsh_launcher_service_h service,
                        tzsh_launcher_service_target_type_e target_type,
                        uint32_t target_res_id,
                        const char* target_path,
                        const char* target_group,
                        tzsh_launcher_service_direction_e direction,
                        int32_t x,
                        int32_t y,
                        const char* shared_widget_info,
                        uint32_t serial);
  static void StopCB(void* data,
                     tzsh_launcher_service_h service,
                     uint32_t serial);
  static void ErrorCB(void* data,
                      tzsh_launcher_service_h service,
                      tzsh_launcher_service_error_e error,
                      uint32_t serial);
  static void CleanupCB(void* data,
                        tzsh_launcher_service_h service,
                        uint32_t serial);

 private:
  IEventListener* listener_ = nullptr;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_LAUNCHER_SERVICE_TZSH_LAUNCHER_SERVICE_INTERNAL_H_
