/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_IMAGE_INTERNAL_H_
#define SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_IMAGE_INTERNAL_H_

#include <string>

#include "screen_connector_common/evas_object.h"
#include "screen_connector_launcher_service/direction.h"
#include "screen_connector_launcher_service/image_type.h"
#include "screen_connector_launcher_service/raw_image.h"

namespace screen_connector {

class Image : public EvasObject {
 public:
  Image(Evas_Object* raw);
  virtual ~Image();

 public:
  void Reset();
  void Update(std::shared_ptr<RawImage> raw_image);
  void SetSize(int32_t width, int32_t height);

  ImageType GetImageType() const;
  Direction GetDirection() const;
  int32_t GetPositionX() const;
  int32_t GetPositionY() const;
  std::string GetSharedWidgetInfo() const;

 private:
  void Update(std::shared_ptr<ImageFile> image_file);
  void Update(std::shared_ptr<WlBuffer> wl_buffer);
  void Update(std::string file_path);
  void Update(std::string file_path, std::string file_group);

 private:
  std::shared_ptr<RawImage> raw_image_;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_IMAGE_INTERNAL_H_
