/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_REMOTE_SURFACE_REMOTE_SURFACE_IMPLEMENTATION_H_
#define SCREEN_CONNECTOR_REMOTE_SURFACE_REMOTE_SURFACE_IMPLEMENTATION_H_

#include <tizen-remote-surface-client-protocol.h>

#include <string>
#include <memory>
#include <map>

#include "screen_connector_remote_surface/buffer_event_interface.h"
#include "screen_connector_remote_surface/common.h"
#include "screen_connector_remote_surface/remote_surface.h"
#include "screen_connector_remote_surface/trs_interface.h"
#include "screen_connector_remote_surface/aul_handle_internal.h"

namespace screen_connector {

class RemoteSurface::Impl : AulHandle::IEventListener, ITRS::IEventListener {
 public:
  void OnAppAdded(const std::string& appId, const std::string& instId,
                  const int pid, const unsigned int surfaceId) override;
  void OnAppRemoved(const std::string& appId, const std::string& instId,
                    const int pid, const unsigned int surfaceId) override;
  void OnAppUpdated(const std::string& appId, const std::string& instId,
                    const int pid, const unsigned int surfaceId) override;
  void OnRawBufferMissing() override;
  void OnRawBufferChanged(int type, std::shared_ptr<WlBuffer> tbm, int fd,
                          uint32_t size, uint32_t time) override;
  void OnInputFilterChanged(uint32_t event_filter) override;

  virtual ~Impl() = default;

 private:
  friend class RemoteSurface;

  Impl(RemoteSurface* parent, const std::string& id, RemoteSurface::Type type,
    std::shared_ptr<WlSurface> wl_surface, bool mock);
  Impl(RemoteSurface* parent, int rid, const std::string& id,
    RemoteSurface::Type type, std::shared_ptr<WlSurface> wl_surface, bool mock);

  void ClearResumingTimer();
  void SetResumingTimer();
  void Redirect();

 private:
  std::string app_id_;
  std::string inst_id_;
  int pid_;
  unsigned int rid_;
  std::unique_ptr<ITRS> surface_;
  std::shared_ptr<WlSurface> wl_surface_;
  std::shared_ptr<WlSurface> bind_surface_;
  std::unique_ptr<AulHandle> aul_handle_;
  RemoteSurface::Type type_;
  bool freeze_ = false;
  bool block_ = false;
  unsigned int resuming_timer_ = 0;
  RemoteSurface::ChangedEventFilter changed_event_filter_ =
      RemoteSurface::FILTER_NONE;
  unsigned int input_event_filter_ = 0;
  bool renderable_ = false;

  static std::map<int, uint32_t> buffer_event_table_;
  static std::map<int, uint32_t> input_type_table_;
  static uint32_t delayed_resuming_time_;
  RemoteSurface* parent_;
  bool mock_;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_REMOTE_SURFACE_REMOTE_SURFACE_IMPLEMENTATION_H_
