/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_REMOTE_SURFACE_WATCHER_IMPLEMENTATION_H_
#define SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_REMOTE_SURFACE_WATCHER_IMPLEMENTATION_H_

#include <string>
#include <memory>
#include <map>
#include <list>

#include "screen_connector_remote_surface_evas/evas_object.h"
#include "screen_connector_remote_surface_evas/watcher_event_interface.h"
#include "screen_connector_remote_surface/remote_surface.h"
#include "screen_connector_remote_surface_evas/remote_surface_mixed_internal.h"
#include "screen_connector_remote_surface_evas/watcher_image_internal.h"
#include "screen_connector_remote_surface/aul_handle_internal.h"

namespace screen_connector {

class RemoteSurfaceWatcher::Impl : AulHandle::IEventListener {
 public:
  virtual ~Impl() = default;
  void OnAppAdded(const std::string& appId, const std::string& instId,
                  const int pid, const unsigned int surfaceId) override;
  void OnAppRemoved(const std::string& appId, const std::string& instId,
                    const int pid, const unsigned int surfaceId) override;
  void OnAppUpdated(const std::string& appId, const std::string& instId,
                    const int pid, const unsigned int surfaceId) override;

 private:
  Impl(RemoteSurfaceWatcher* parent, RemoteSurface::Type type,
      std::shared_ptr<EvasObject> viewer_win);

 private:
  friend class RemoteSurfaceWatcher;

  std::shared_ptr<EvasObject> viewer_win_;
  std::list<std::unique_ptr<RemoteSurfaceMixed>> surfaces_;
  std::unique_ptr<AulHandle> aul_handle_;
  RemoteSurfaceWatcher* parent_;
};

}  // namespace screen_connector


#endif  // SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_REMOTE_SURFACE_WATCHER_IMPLEMENTATION_H_
