/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "screen_connector_shared_widget_launch/tzsh_shared_widget_launch_internal.h"
#include "screen_connector_shared_widget_launch/log_internal.h"

namespace screen_connector {

TzshSharedWidgetLaunch::TzshSharedWidgetLaunch(tzsh_shared_widget_launch_h raw)
  : Handle<tzsh_shared_widget_launch_h>(raw, true) {
}

TzshSharedWidgetLaunch::~TzshSharedWidgetLaunch() {
  tzsh_shared_widget_launch_destroy(GetRaw());
}

tzsh_shared_widget_launch_h TzshSharedWidgetLaunch::GetHandle() const {
  return GetRaw();
}

void TzshSharedWidgetLaunch::SetListener(
    TzshSharedWidgetLaunch::IEventListener* listener) {
  listener_ = listener;
  tzsh_shared_widget_launch_prepare_cb_set(GetRaw(), PrepareCB, listener_);
}

int TzshSharedWidgetLaunch::PrepareDone(const std::string& shared_widget_info,
    PrepareState state,
    uint32_t serial) {
  return tzsh_shared_widget_launch_prepare_done(GetRaw(),
      shared_widget_info.c_str(),
      static_cast<tzsh_shared_widget_launch_prepare_state_e>(state),
      serial);
}

void TzshSharedWidgetLaunch::PrepareCB(void* data,
    tzsh_shared_widget_launch_h tzsh_swl,
    tzsh_shared_widget_launch_prepare_state_e state,
    uint32_t serial) {
  auto listener = static_cast<IEventListener*>(data);
  listener->OnPrepareEvent(static_cast<PrepareState>(state), serial);
}

}  // namespace screen_connector
