/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_SHARED_WIDGET_LAUNCH_TZSH_SHARED_WIDGET_LAUNCH_INTERNAL_H_
#define SCREEN_CONNECTOR_SHARED_WIDGET_LAUNCH_TZSH_SHARED_WIDGET_LAUNCH_INTERNAL_H_

#include <stdint.h>
#include <tzsh_shared_widget_launch.h>
#include <unistd.h>

#include <memory>
#include <string>

#include "screen_connector_common/handle.h"
#include "screen_connector_shared_widget_launch/prepare_state.h"

namespace screen_connector {

class TzshSharedWidgetLaunch : public Handle<tzsh_shared_widget_launch_h> {
 public:
  class IEventListener {
   public:
    virtual void OnPrepareEvent(PrepareState state, uint32_t serial) = 0;
  };

  explicit TzshSharedWidgetLaunch(tzsh_shared_widget_launch_h raw);
  virtual ~TzshSharedWidgetLaunch();

  tzsh_shared_widget_launch_h GetHandle() const;
  void SetListener(IEventListener* listener);

  int PrepareDone(const std::string& shared_widget_info,
                  PrepareState state,
                  uint32_t serial);

 private:
  static void PrepareCB(void* data,
                        tzsh_shared_widget_launch_h tzsh_swl,
                        tzsh_shared_widget_launch_prepare_state_e state,
                        uint32_t serial);

 private:
  IEventListener* listener_ = nullptr;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_SHARED_WIDGET_LAUNCH_TZSH_SHARED_WIDGET_LAUNCH_INTERNAL_H_
